#! /usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import gtk
import random

ui_str = """<ui>
    <menubar name="MenuBar">
        <menu action="File">
            <menuitem action="open"/>
            <separator/>
            <menuitem action="play"/>
            <menuitem action="stop"/>
            <menuitem action="prev"/>
            <menuitem action="next"/>
            <separator/>
            <menu action="Rep">
                <menuitem action="rep_non"/>
                <menuitem action="rep_one"/>
                <menuitem action="rep_all"/>
                <menuitem action="rep_rdm"/>
            </menu>
            <menu action="asp">
                <menuitem action="asp0"/>
                <menuitem action="asp1"/>
                <menuitem action="asp2"/>
                <menuitem action="asp3"/>
            </menu>
            <separator/>
            <menuitem action="quit"/>
        </menu>
        <menu action="View">
            <menuitem action="full"/>
            <separator/>
            <menuitem action="x00"/>
            <menuitem action="x10"/>
            <menuitem action="x20"/>
            <menuitem action="x30"/>
            <menuitem action="x05"/>
            <menuitem action="x15"/>
            <menuitem action="x25"/>
            <separator/>
            <menuitem action="xs1"/>
            <menuitem action="xs2"/>
            <menuitem action="xs3"/>
        </menu>
        <menu action="Setting">
            <menuitem action="set"/>
        </menu>
        <menu action="Help">
            <menuitem action="ver"/>
        </menu>
    </menubar>
    <popup name="pop_main">
        <menuitem action="open"/>
        <separator/>
        <menu action="View">
            <menuitem action="full"/>
            <separator/>
            <menuitem action="x00"/>
            <menuitem action="x10"/>
            <menuitem action="x20"/>
            <menuitem action="x30"/>
            <menuitem action="x05"/>
            <menuitem action="x15"/>
            <menuitem action="x25"/>
            <separator/>
            <menuitem action="xs1"/>
            <menuitem action="xs2"/>
            <menuitem action="xs3"/>
        </menu>
        <menu action="Rep">
            <menuitem action="rep_non"/>
            <menuitem action="rep_one"/>
            <menuitem action="rep_all"/>
            <menuitem action="rep_rdm"/>
        </menu>
        <menu action="asp">
            <menuitem action="asp0"/>
            <menuitem action="asp1"/>
            <menuitem action="asp2"/>
            <menuitem action="asp3"/>
        </menu>
        <separator/>
        <menuitem action="play"/>
        <menuitem action="stop"/>
        <menuitem action="prev"/>
        <menuitem action="next"/>
        <separator/>
        <menuitem action="set"/>
        <separator/>
        <menuitem action="quit"/>
    </popup>
    <popup name="pop_list">
        <menuitem action="list_close"/>
    </popup>
</ui>"""

size_str = ["win","x1.0","x2.0","x3.0","x0.5","x1.5","x2.5","set1","set2","set3"]
ac1_str = ["x00","x10","x20","x30","x05","x15","x25","xs1","xs2","xs3"]
ac2_str = ["rep_non", "rep_one", "rep_all", "rep_rdm"]
ac3_str = ["asp0", "asp1", "asp2", "asp3"]
asp_str = ["default", "4:3", "16:9", "custom"]
MIMES = "video/x-ms-wmv|video/x-flv|video/mp4|video/mpeg|video/x-msvideo"

def clamp(a,b,c): return min(max(a,b),c)

#
# StatusBar
#

class CStatusBar(gtk.VBox):
    """
        Instead GtkStatusbar
    """
    def __init__(self, owner, arg):
        """
            @param arg: list of string
              labels Initial value
        """
        self._win = owner.w
        self._owner = owner
        # public menber
        self.label = []
        # init Original inheritance
        gtk.VBox.__init__(self)
        # GtkLabel and GtkVSeparator
        hb = gtk.HBox()
        for s in arg:
            l = gtk.Label(s)
            p = gtk.VSeparator()
            hb.pack_start(l, False, False, 10)
            hb.pack_start(p, False, False, 0)
            self.label.append(l)
        # Dummy
        ld = gtk.Label()
        hb.pack_start(ld)
        # Grip
        grip = gtk.DrawingArea()
        grip.set_size_request(16, 16)
        grip.set_events(gtk.gdk.EXPOSURE_MASK | gtk.gdk.BUTTON_PRESS_MASK)
        grip.connect("expose-event", self.__on_grip_expose)
        grip.connect("button-press-event", self.__on_resize)
        grip.connect("realize", self.__on_realize)
        hb.pack_end(grip, False, False, 0)
        # pack
        s0 = gtk.HSeparator()
        self.pack_start(s0, False, False, 0)
        self.pack_start(hb, False, False, 0)
        self.show_all()

    def __on_resize(self, widget, event):
        """
            begin resize @ Toplevel Window
        """
        self._owner.hand_resize()
        self._win.begin_resize_drag(
                gtk.gdk.WINDOW_EDGE_SOUTH_EAST,
                event.button,
                int(event.x_root),
                int(event.y_root),
                event.time
            )

    def __on_realize(self, widget, event=None):
        """
            cursor change
        """
        cur_grip = gtk.gdk.Cursor(gtk.gdk.BOTTOM_RIGHT_CORNER)
        widget.window.set_cursor(cur_grip)

    def __on_grip_expose(self, widget, event):
        """
            Draw Resize Grip
        """
        widget.style.paint_resize_grip(
                widget.window,
                gtk.STATE_NORMAL,
                None,
                None,
                None,
                gtk.gdk.WINDOW_EDGE_SOUTH_EAST,
                0,
                0,
                16,
                16
            )

#
# ListBox
#

class CListBox(gtk.HBox):
    """
        GtkTreeView wrapping panels
    """
    def __init__(self, window):
        """
            @param window: Toplevel Window
              Synchronize Resize
        """
        self._win = window
        gtk.HBox.__init__(self)
        # private:
        self.__resize_w = 0
        self.__resize_x = 0
        self.__resize_y = 0
        self.__cx = 0
        self.__cy = 0
        # partition
        vs = gtk.VSeparator()
        vs.set_size_request(5, -1)
        self.pack_start(vs, False, True, 0)
        # on ScrolledWindow TreeView
        self.sw = gtk.ScrolledWindow()
        self.sw.set_size_request(100, -1)
        self.sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)#gtk.POLICY_ALWAYS
        self.sw.set_shadow_type(gtk.SHADOW_IN)
        # GtkTreeView
        self.view = gtk.TreeView()
        self.sw.add(self.view)
        # GtkTreeColumn
        self.column1 = gtk.TreeViewColumn()
        self.view.append_column(self.column1)
        self.connect("size-request", self.__on_view_size_request)
        cell1 = gtk.CellRendererText()
        self.column1.pack_start(cell1)
        self.column1.add_attribute(cell1, "text", 0)
        # liststore create  
        self.liststore = gtk.ListStore(str)
        self.view.set_model(self.liststore)
        self.view.set_headers_visible(False)
        self.selection = self.view.get_selection()
        self.selection.set_mode(gtk.SELECTION_SINGLE)
        # treeview pack
        self.pack_start(self.sw)#, False, True, 0)
        # DrawingArea
        self.dwa = gtk.DrawingArea()
        self.dwa.set_events(gtk.gdk.EXPOSURE_MASK | 
                            gtk.gdk.POINTER_MOTION_MASK | 
                            gtk.gdk.POINTER_MOTION_HINT_MASK | 
                            gtk.gdk.BUTTON_PRESS_MASK | 
                            gtk.gdk.BUTTON_RELEASE_MASK | 
                            gtk.gdk.STRUCTURE_MASK )
                            #gtk.gdk.VISIBILITY_NOTIFY_MASK )
        self.dwa.set_size_request(10, -1)
        self.pack_end(self.dwa, False)
        # Connect
        self.dwa.connect("button-press-event", self.__on_button_press_event)
        self.dwa.connect("motion-notify-event", self.__on_motion_notify_event)
        self.dwa.connect("button-release-event", self.__on_button_release_event)
        self.dwa.connect("expose-event", self.__on_expose_event)
        self.dwa.connect("realize", self.__on_realize)
        #self.liststore.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self.show_all()

    def is_text(self, text):
        #return text in self.liststore[0]
        #print [ r[0] for r in self.liststore ]
        return text in [ r[0] for r in self.liststore ]

    def select_text(self, text):
        model = self.view.get_model()
        it = model.get_iter_first()
        if it == None:
            self.selection.unselect_all()
        else:
            self._n = 0
            while 1:
                s = model.get_value(it, 0)
                if s == text:
                    self.selection.select_iter(it)
                    break
                it = model.iter_next(it)
                if it == None: break

    def selection_scroll(self):
        """
            def __on_selection_changed(self, selection):
            を利用だと上手くいかない
        """
        # Get GtkAdjustment
        adjustment = self.view.get_vadjustment()
        # Is Select Check
        n = self.get_select_num()
        if n == -1:
            return
        page_size = adjustment.get_property("page_size")
        column_height = self.column1.cell_get_size(gtk.gdk.Rectangle())[4]
        # 計算値
        sc = column_height * n
        # into Rect
        v = adjustment.get_value()
        h = page_size + v - (column_height * 2)
        if sc == 0.0:
            adjustment.set_value(sc)
        elif sc < v:
            adjustment.set_value(sc - column_height)
        elif sc >= h:
            adjustment.set_value(sc - page_size + (column_height * 2))

    def get_select_num(self):
        model, it = self.selection.get_selected()
        if it:
            return model.get_path(it)[0]
        return -1

    def __on_view_size_request(self, widget, event=None):
        self.selection_scroll()

    def __on_button_press_event(self, widget, event):
        # begin resize
        if event.button == 1:
            self.__resize_w = self.get_property("width_request")
            self.__resize_x = event.x_root
            self.__cx, self.__cy = self._win.get_size()

    def __on_motion_notify_event(self, widget, event):
        # size change
        if self.__resize_w:
            x = event.x_root - self.__resize_x
            if self.__resize_w + x > 0:
                self.set_property("width_request", self.__resize_w + x)
                self._win.resize(int(self.__cx + x), int(self.__cy))

    def __on_button_release_event(self, widget, event):
        # end resize
        if event.button == 1:
            self.__resize_w = 0
            self.dwa.queue_draw()

    def __on_expose_event(self, widget, event):
        # GetDC()（Windows)
        gc = widget.style.fg_gc[gtk.STATE_NORMAL]
        # Draw Vertical Line
        for i in range(10):
            if i%3 == 0:
                widget.window.draw_line(gc, i, 0, i, widget.allocation.height)

    def __on_realize(self, widget, event=None):
        cur_grip = gtk.gdk.Cursor(gtk.gdk.RIGHT_TEE)#.SB_H_DOUBLE_ARROW)
        self.dwa.window.set_cursor(cur_grip)


#
# ToolBox
#

class CToolBox(gtk.VBox):
    """
        ToolBox
    """
    def __init__(self):
        gtk.VBox.__init__(self)
        # SeekBar
        self.seekadj = gtk.Adjustment(0.0, 0.0, 100.0, 1.0, 1.0, 1.0)
        self.seekbar = gtk.HScale(self.seekadj)
        self.seekbar.set_events( gtk.gdk.BUTTON_PRESS_MASK | 
                                gtk.gdk.BUTTON_RELEASE_MASK | 
                                gtk.gdk.STRUCTURE_MASK )
        self.seekbar.set_draw_value(False)
        # ctrl pack
        self.hbox = gtk.HBox()
        # images
        self._path = os.path.dirname( __file__ )
        self.img = []
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/pausebutton.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/stopbutton.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/playlist.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/rep_one.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/rep_all.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/rep_rdm.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/pausebutton.xpm")))
        self.img.append(gtk.image_new_from_file(os.path.join(self._path, "img/playbutton.xpm")))
        # button
        self.__button = []
        for i in range(6):
            if i < 3:
                b = gtk.Button()
            else:
                b = gtk.ToggleButton()
            b.set_focus_on_click(False)   # no focus
            b.set_relief(gtk.RELIEF_NONE) # Flat Button
            b.add(self.img[i])
            self.__button.append(b)
        self.play = self.__button[0]
        self.stop = self.__button[1]
        self.playlist = self.__button[2]
        self.one = self.__button[3]
        self.all = self.__button[4]
        self.rdm = self.__button[5]
        # volume bar
        self.volumeadj = gtk.Adjustment(50.0, 0.0, 101.0, -1.0, 1.0, 1.0)
        self.volumebar = gtk.HScale(self.volumeadj)
        self.volumebar.set_value_pos(gtk.POS_RIGHT)
        self.volumebar.set_size_request(100, -1)
        self.volumebar.set_digits(0)
        self.volumebar.set_show_fill_level(True)
        self.volumebar.set_events(  gtk.gdk.BUTTON_PRESS_MASK | 
                                    gtk.gdk.BUTTON_RELEASE_MASK | 
                                    gtk.gdk.STRUCTURE_MASK )
        # packing
        self.hbox.pack_start(self.play, False, False)
        self.hbox.pack_start(self.stop, False, False)
        self.hbox.pack_start(self.volumebar, False, False)
        self.hbox.pack_end(self.playlist, False, False)
        self.hbox.pack_end(self.rdm, False, False)
        self.hbox.pack_end(self.all, False, False)
        self.hbox.pack_end(self.one, False, False)
        self.pack_start(self.seekbar)
        self.pack_start(self.hbox, False, False)
        self.num = 0
        #self.set_sensitive(False)
        self.set_enable_ctrl(False)
        self.show_all()

    #def set_volume(self, num):
    #    self.volumeadj.set_value(num)

    def change_img(self, num):
        if self.num == num:
            return
        if num == 1:
            self.img[0].set_from_pixbuf(self.img[7].get_pixbuf())
        else:
            self.img[0].set_from_pixbuf(self.img[6].get_pixbuf())
        self.num = num

    def set_enable_ctrl(self, b):
        self.play.set_sensitive(b)
        self.stop.set_sensitive(b)
        self.seekbar.set_sensitive(b)

class CRList():
    """
        Random File List
    """
    def __init__(self):
        self.l = []
        self.m = []

    def get(self):
        for s in self.l:
            if not s in self.m:
                self.m.append(s)
                return s
        self.m[:] = []
        return self.l[0]

    def append(self, text):
        self.l.append(text)

    def clear(self):
        self.l[:] = []
        self.m[:] = []

    def shuffle(self):
        random.shuffle(self.l)

class CFullCtrl(gtk.Window):
    """
        Full Screen Controrer
    """
    def __init__(self, parent, box, parent2, bar):
        """
            GtkWindow at GTK_WINDOW_POPUP
        """
        gtk.Window.__init__(self, gtk.WINDOW_POPUP)
        # memory pointer
        self.p1 = parent
        self.p2 = parent2
        self.box = box
        self.bar = bar
        # Get Desktop Window Geometry
        root = gtk.gdk.get_default_root_window()
        x, y, w, h, d = root.get_geometry()
        # move and resize
        self.set_size_request(w, -1)
        self.y_pos = h - box.allocation.height - bar.allocation.height
        self.move(0, self.y_pos)
        # Change parent
        vbox = gtk.VBox()
        self.box.reparent(vbox)
        self.bar.reparent(vbox)
        vbox.show_all()
        self.add(vbox)
        self.connect("destroy", self.on_destroy)

    def on_destroy(self, widget, event=None):
        """
            Return Parent
        """
        self.box.reparent(self.p1)
        self.p1.set_child_packing(self.box, False, True, 0, gtk.PACK_END)
        self.bar.reparent(self.p2)
        self.p2.set_child_packing(self.bar, False, False, 0, gtk.PACK_START)
        return True

class CVideoArea(gtk.DrawingArea):
    """
        Movie Draw Area
    """
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.connect('expose-event', self.on_expose_event)

    def on_expose_event(self, widget, event):
        self.xid = widget.window.xid
        return False

#
# Structs (Windows compatible)
#

class CSize(object):
    __slots__ = ["width", "height"]
    def __init__(self, w=0, h=0):
        self.width = w
        self.height = h

class CPoint(object):
    __slots__ = ["x", "y"]
    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

class CGesture(object):
    __slots__ = ["start", "result", "title", "prevchar", "x", "y"]
    def __init__(self):
        self.start = False
        self.result = ""
        self.title = ""
        self.prevchar = ""
        self.x = 0
        self.y = 0

class CSetting(object):
    __slots__ = [
            "mimes", "position", "init_size_on", "init_size_val", "esc_exit",
            "mem_rep", "mem_aspect", "endressloop", "severity", "aspect_rate",
            "set_size"]
    def __init__(self):
        self.mimes = MIMES
        self.position = False
        self.init_size_on = False
        self.init_size_val = 0
        self.esc_exit = False
        self.mem_rep = False
        self.mem_aspect = False
        self.endressloop = False
        self.severity = 25
        self.aspect_rate = [1,1]
        self.set_size = [[320,240],[640,480],[1280,720]]

class CSettingWin(object):
    __slots__ = [
            "list_width", "list_show", "volume", "video_size", "repeat",
            "aspect", "repeat_mem", "aspect_mem"]
    def __init__(self):
        self.list_width = 100
        self.list_show = True
        self.volume = 100.0
        self.video_size = 0
        self.repeat = 0
        self.aspect = 0
        self.repeat_mem = 0
        self.aspect_mem = 0

if __name__ == "__main__":
    # Unconscientious debug
    import y901main
    y901main.main()
