#!/usr/bin/env python
#-*- coding:utf-8 -*-

import pygtk
pygtk.require("2.0")
import gtk
import pygst
pygst.require('0.10')
import gst
import gobject
import os
import sys
import dbus
import dbus.service
import dbus.glib
import urllib
import random
import gio
import gnomevfs
import glib
# Included Module
import inifile8
import y901dialog
from common import *
import numsort

version = "0.3.2"

class Y901window(dbus.service.Object):
    """
        Main Window
    """
    def __init__(self, bus_name):
        """
            @param bus_name: dbus name
        """
        # want dbus.glib
        dbus.service.Object.__init__(self, bus_name, "/org/y901x/Object1")
        sys.path.insert(0, os.path.dirname(__file__))
        self.w = gtk.Window()
        # attribute
        self.is_playing = False
        self.p_path = ""
        self.p_filename = ""
        self.seek_at_play = False
        self.is_seek = False
        self.timer_id = -1
        self.p_position = gst.CLOCK_TIME_NONE
        self.p_duration = gst.CLOCK_TIME_NONE
        #self.resize_list = 0
        self.rlist = CRList()
        self.fullscreen = False
        self.fullsize = CSize(320, 240)
        self.fullobj = None
        self.full_list = False
        self.settingwin = CSettingWin()
        self.setting = CSetting()
        self.mime_list = []
        self.gesture = CGesture()
        self.vsize = CSize(1, 1)
        self.mousepoint = CPoint()
        self.invisiblecount = 0;
        # Read config file
        inipath = os.path.join(os.environ["HOME"], ".y901x")
        if os.path.exists(inipath):
            inipath = os.path.join(inipath, "y901x.ini")
            if os.path.exists(inipath):
                ini = inifile8.Inifile(inipath)
                try:
                    # [window]
                    x = ini.read_int("window", "left", 0)
                    y = ini.read_int("window", "top", 0)
                    cx = ini.read_int("window", "width", 320)
                    cy = ini.read_int("window", "height", 320)
                    self.settingwin.list_width  = ini.read_int("window", "list_width", 100)
                    self.settingwin.list_show   = ini.read_bool("window", "list_show", True)
                    self.settingwin.volume      = ini.read_float("window", "volume", 100.0)
                    self.settingwin.video_size  = ini.read_int("window", "video_size", 0)
                    self.settingwin.repeat_mem  = ini.read_int("window", "repeat", 0)
                    self.settingwin.aspect_mem  = ini.read_int("window", "aspect", 0)
                    # [setting]
                    self.setting.position       = ini.read_bool("setting", "position", False)
                    self.setting.init_size_on   = ini.read_bool("setting", "init_size_on", False)
                    self.setting.init_size_val  = ini.read_int("setting", "init_size_val", 0)
                    self.setting.esc_exit       = ini.read_bool("setting", "esc_exit", False)
                    self.setting.mem_rep        = ini.read_bool("setting", "mem_rep", False)
                    self.setting.mem_aspect     = ini.read_bool("setting", "mem_aspect", False)
                    self.setting.endressloop    = ini.read_bool("setting", "loop", False)
                    self.setting.severity       = ini.read_int("setting", "severity", 25)
                    self.setting.mimes          = ini.read_str("setting", "mimes", MIMES)
                    self.setting.set_size[0][0] = ini.read_int("setting", "set_size1", 320)
                    self.setting.set_size[0][1] = ini.read_int("setting", "set_size2", 240)
                    self.setting.set_size[1][0] = ini.read_int("setting", "set_size3", 640)
                    self.setting.set_size[1][1] = ini.read_int("setting", "set_size4", 480)
                    self.setting.set_size[2][0] = ini.read_int("setting", "set_size5", 1280)
                    self.setting.set_size[2][1] = ini.read_int("setting", "set_size6", 720)
                    self.setting.aspect_rate[0] = ini.read_int("setting", "aspect_rate0", 1)
                    self.setting.aspect_rate[1] = ini.read_int("setting", "aspect_rate1", 1)
                except Exception, s:
                    self.messagebox("%s" % s)
                    sys.exit()
                # resize and move
                if self.setting.init_size_on:
                    self.settingwin.video_size = self.setting.init_size_val
                self.w.resize(cx, cy)
                if self.setting.position:
                    self.w.move(x, y)
        # List up the MIME Types
        self.mime_list = self.setting.mimes.split("|")
        # Create GtkUIManager
        self.uimanager = gtk.UIManager()
        # GtkAccelGroup (accel_key, accel_mods, accel_flags, callback)
        self.accelgroup0 = self.uimanager.get_accel_group()
        self.accelgroup0.connect_group(gtk.keysyms.KP_0, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_1, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_2, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_3, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_4, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_5, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_6, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_7, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_8, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_9, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.F12, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.F5, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.F2, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.Escape, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.Delete, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.Delete, gtk.gdk.SHIFT_MASK, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.Delete, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.w.add_accel_group(self.accelgroup0)
        # Create GtkActionGroup
        self.actiongroup = gtk.ActionGroup("sasakimamenu")
        # GtkActionEntry (name, stock_id, label, accelerator, tooltip, callback)
        # (warning: Up, Down key is Don't work)
        self.ac0 = [("open", gtk.STOCK_OPEN, "開く(_O)", "<Control>O", "開く", self.on_open),
                    ("play", gtk.STOCK_MEDIA_PLAY, "再生/ポーズ(_P)", "space", "再生/ポーズ", self.on_play),
                    ("stop", gtk.STOCK_MEDIA_STOP, "停止(_S)", "BackSpace", "停止", self.on_stop),
                    ("prev", gtk.STOCK_GO_UP, "手前を再生(_U)", "Up", "手前を再生", self.on_prev),
                    ("next", gtk.STOCK_GO_DOWN, "次を再生(_D)", "Down", "次を再生", self.on_next),
                    ("quit", gtk.STOCK_QUIT, "終了(_Q)", "<Control>Q", "さいなら", self.on_quit),
                    ("set", gtk.STOCK_PREFERENCES, "設定(_P)", "P", "設定", self.on_setting),
                    ("ver", gtk.STOCK_ABOUT, "情報(_B)", "<Control>B", "情報", self.on_about),
                    ("full", None, "フルスクリーン(_F)", "Z", "フルスクリーン切り替え", self.on_full),
                    ("File", None, "ファイル(_F)"),
                    ("Rep", None, "リピート(_R)"),
                    ("asp", None, "アスペクト比(_A)"),
                    ("View", None, "表示(_V)"),
                    ("Setting", None, "設定(_P)"),
                    ("Help", None, "ヘルプ(_H)")]
        self.actiongroup.add_actions(self.ac0)
        self.ac4 = [("list_close", None, "リストを閉じる", None, "リストを閉じる", self.on_list_button_clicked)]
        self.actiongroup.add_actions(self.ac4)
        # GtkRadioActionEntry (name, stock_id, label, accelerator, tooltip, value)
        self.ac1 = [("x00", None, "ウインドサイズ", "0", "ウインドサイズ", 0),
                    ("x10", None, "x1.0", "1", "x1.0", 1),
                    ("x20", None, "x2.0", "2", "x2.0", 2),
                    ("x30", None, "x3.0", "3", "x3.0", 3),
                    ("x05", None, "x0.5", "4", "x0.5", 4),
                    ("x15", None, "x1.5", "5", "x1.5", 5),
                    ("x25", None, "x2.5", "6", "x2.5", 6),
                    ("xs1", None, "set1", "7", "set1", 7),
                    ("xs2", None, "set2", "8", "set1", 8),
                    ("xs3", None, "set3", "9", "set1", 9) ]
        self.ac2 = [("rep_non", None, "リピート無し", "F6", "リピート無し", 0),
                    ("rep_one", None, "シングルリピート", "F7", "シングルリピート", 1),
                    ("rep_all", None, "すべてリピート", "F8", "すべてリピート", 2),
                    ("rep_rdm", None, "ランダム再生", "F9", "ランダム再生", 3) ]
        self.ac3 = [("asp0", None, "default", "<Control>A", "アスペクト比 default", 0),
                    ("asp1", None, "4:3", "<Control>S", "アスペクト比 4:3", 1),
                    ("asp2", None, "16:9", "<Control>D", "アスペクト比 16:9", 2),
                    ("asp3", None, "custom", "<Control>F", "アスペクト比 custom", 3) ]
        # GtkRadioAction (entries, value=0, on_change=None, user_data=None)
        self.actiongroup.add_radio_actions(self.ac1, -1, self.on_size_change)
        self.actiongroup.add_radio_actions(self.ac2, 0, self.on_rep_change)
        self.actiongroup.add_radio_actions(self.ac3, 0, self.on_asp_change)
        # Update the GtkUIManager
        self.uimanager.insert_action_group(self.actiongroup, 0)
        self.uimanager.add_ui_from_string(ui_str)
        self.uimanager.ensure_update()
        # Get the PopupMenu
        self.popup_menu = self.uimanager.get_widget('/pop_main')
        # StatusBar
        self.statusbar = CStatusBar(self, ["00:00/00:00", "win", "defaule"])
        # ListBox
        self.listbox = CListBox(self.w)
        self.listbox.set_size_request(self.settingwin.list_width, -1)
        self.listbox.view.connect("button_release_event", self.on_listbox_button_release)
        self.listbox.view.connect("button_press_event", self.on_listbox_button_press)
        self.listbox.set_property("visible", self.settingwin.list_show)
        self.listpopup = self.uimanager.get_widget('/pop_list')
        # ToolBox
        self.toolbox = CToolBox()
        self.toolbox.play.connect("clicked", self.on_play)
        self.toolbox.stop.connect("clicked", self.on_stop)
        self.toolbox.playlist.connect("clicked", self.on_list_button_clicked)
        self.toolbox.seekbar.connect("value_changed", self.on_seek_value_changed)
        self.toolbox.seekbar.connect("button_press_event", self.on_seek_button_press_event)
        self.toolbox.seekbar.connect("button_release_event", self.on_seek_button_release_event)
        self.toolbox.volumebar.connect("button_release_event", self.on_volume_button_release_event)
        self.toolbox.one.connect("clicked", self.on_rep_btn, 1)
        self.toolbox.all.connect("clicked", self.on_rep_btn, 2)
        self.toolbox.rdm.connect("clicked", self.on_rep_btn, 3)
        self.toolbox.volumeadj.set_value(self.settingwin.volume)
        self.toolbox.volumebar.set_property("sensitive", False)
        # PlayBin2
        self.player = gst.element_factory_make("playbin2", "player")
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.gst_on_message)
        bus.connect("sync-message::element", self.gst_on_sync_message)
        # Container
        vbox_submain = gtk.HBox()
        vbox_submain.show()
        self.hbox_ctrl = gtk.VBox()
        self.hbox_ctrl.show()
        self.vbox_main = gtk.VBox()
        self.vbox_main.show()
        # Video Window, Background Color to Black
        self.video_window = CVideoArea()
        self.video_window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(0,0,0))
        self.video_window.set_property("can_focus", True)
        # GtkAspectFrame, expose-event a Background Color to Black
        self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0, obey_child=False)
        self.aframe.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(0,0,0))
        self.aframe.set_shadow_type(gtk.SHADOW_NONE)
        self.aframe.set_events(gtk.gdk.EXPOSURE_MASK | gtk.gdk.STRUCTURE_MASK | gtk.gdk.VISIBILITY_NOTIFY_MASK)
        self.aframe.connect("expose-event", self.on_aframe_expose)
        self.aframe.add(self.video_window)
        # menubar
        self.menubar = self.uimanager.get_widget("/MenuBar")
        # packing
        self.hbox_ctrl.pack_start(self.menubar, False)
        self.hbox_ctrl.pack_start(self.aframe)
        self.hbox_ctrl.pack_end(self.toolbox, False)
        self.hbox_ctrl.show_all()
        vbox_submain.pack_start(self.hbox_ctrl)
        vbox_submain.pack_end(self.listbox, False)
        self.vbox_main.pack_start(vbox_submain)
        self.vbox_main.pack_start(self.statusbar, False, False, 0)
        self.w.add(self.vbox_main)
        # TitleBar
        iconpath = os.path.join(os.path.dirname(__file__), "y901x.xpm")
        self.icon = gtk.gdk.pixbuf_new_from_file(iconpath)
        self.w.set_icon(self.icon)
        self.w.set_title("Y901x")
        # Drop source set
        dnd_list = [("text/uri-list", 0, 0)]
        self.w.drag_dest_set(
                gtk.DEST_DEFAULT_MOTION |
                gtk.DEST_DEFAULT_HIGHLIGHT |
                gtk.DEST_DEFAULT_DROP,
                dnd_list, gtk.gdk.ACTION_MOVE )
        # GDK event set
        self.w.set_events(
                gtk.gdk.KEY_PRESS_MASK |
                gtk.gdk.KEY_RELEASE_MASK |
                gtk.gdk.STRUCTURE_MASK )
        self.video_window.set_events(
                gtk.gdk.LEAVE_NOTIFY_MASK |
                gtk.gdk.BUTTON_PRESS_MASK |
                gtk.gdk.BUTTON_RELEASE_MASK |
                gtk.gdk.POINTER_MOTION_MASK |
                gtk.gdk.POINTER_MOTION_HINT_MASK )
        # Create invisible cursor
        pixmap = gtk.gdk.Pixmap(None, 1, 1, 1)
        color = gtk.gdk.Color()
        self.noncursor = gtk.gdk.Cursor(pixmap, pixmap, color, color, 0, 0)
        # Signals
        self.video_window.connect("button_press_event", self.on_button_press)
        self.video_window.connect("button_release_event", self.on_button_release)
        self.video_window.connect("motion_notify_event", self.on_motion_notify)
        self.w.connect("drag_data_received", self.on_drag_data_received)
        self.w.connect("key_press_event", self.on_key_press)
        self.w.connect("key_release_event", self.on_key_release)
        self.w.connect("delete-event", self.on_quit)
        self.w.connect("show", self.on_show)
        self.w.show()

    def on_show(self, widget, event=None):
        """
            After the Show
        """
        # Radio menu position changes. Throw on_size_change signal.
        action = self.actiongroup.get_action(ac1_str[self.settingwin.video_size])
        action.set_current_value(self.settingwin.video_size)
        # Repert memory Attach
        if self.setting.mem_rep:
            action2 = self.actiongroup.get_action(ac2_str[self.settingwin.repeat_mem])
            action2.set_current_value(self.settingwin.repeat_mem)
        # Aspect rate Attach
        if self.setting.mem_aspect:
            action3 = self.actiongroup.get_action(ac3_str[self.settingwin.aspect_mem])
            action3.set_current_value(self.settingwin.aspect_mem)
        # move focus
        self.w.set_focus(self.video_window)
        # change gstreamer volume
        self.toolbox.volumebar.connect("value_changed", self.on_volume_value_changed)
        # argv processing
        if len(sys.argv) > 1:
            if "//" in sys.argv[1]:
                self.set_uri(sys.argv[1])
            else:
                self.set_uri("file://" + os.path.abspath(sys.argv[1]))


########################################################################
#
#                       Functions
#
########################################################################


    def messagebox(self, text, icon=gtk.MESSAGE_WARNING, button=gtk.BUTTONS_OK):
        dlg = gtk.MessageDialog(self.w, gtk.DIALOG_MODAL, icon, button, text)
        dlg.set_title("from Y901x")
        r = dlg.run()
        dlg.destroy()
        return r

    @dbus.service.method("org.y901x.Interface1")
    def set_uri(self, uri):
        """
            Once NULL.
            Non supported file => gst_on_message
        """
        if uri == None:
            return
        self.clear()
        self.player.set_state(gst.STATE_NULL)
        self.toolbox.set_enable_ctrl(False)
        while gtk.events_pending():
            gtk.main_iteration()
        self.player.set_property("uri", uri)
        self.player.set_state(gst.STATE_PLAYING)
        self.w.set_focus(self.video_window)

    def close(self):
        self.player.set_state(gst.STATE_NULL)
        inipath = os.path.join(os.environ["HOME"], ".y901x")
        if not os.path.exists(inipath):
            os.mkdir(inipath)
        inipath = os.path.join(inipath, "y901x.ini")
        ini = inifile8.Inifile(inipath)
        try:
            # [window]
            x, y = self.w.get_position()
            ini.write_int("window", "left", x)
            ini.write_int("window", "top", y)
            if self.fullscreen:
                ini.write_int("window", "width", self.fullsize.width)
                ini.write_int("window", "height", self.fullsize.height)
            else:
                cx, cy = self.w.get_size()
                ini.write_int("window", "width", cx)
                ini.write_int("window", "height", cy)
            ini.write_int("window", "list_width", self.listbox.get_property("width_request"))
            ini.write_bool("window", "list_show", self.listbox.get_property("visible"))
            ini.write_float("window", "volume", self.toolbox.volumeadj.get_value())
            ini.write_int("window", "video_size", self.settingwin.video_size)
            ini.write_int("window", "repeat", self.settingwin.repeat)
            ini.write_int("window", "aspect", self.settingwin.aspect)
            # [setting]
            ini.write_bool("setting", "position", self.setting.position)
            ini.write_bool("setting", "init_size_on", self.setting.init_size_on)
            ini.write_int("setting", "init_size_val", self.setting.init_size_val)
            ini.write_bool("setting", "esc_exit", self.setting.esc_exit)
            ini.write_bool("setting", "mem_rep", self.setting.mem_rep)
            ini.write_bool("setting", "mem_aspect", self.setting.mem_aspect)
            ini.write_bool("setting", "loop", self.setting.endressloop)
            ini.write_int("setting", "severity", self.setting.severity)
            ini.write_str("setting", "mimes", self.setting.mimes)
            ini.write_int("setting", "set_size1", self.setting.set_size[0][0])
            ini.write_int("setting", "set_size2", self.setting.set_size[0][1])
            ini.write_int("setting", "set_size3", self.setting.set_size[1][0])
            ini.write_int("setting", "set_size4", self.setting.set_size[1][1])
            ini.write_int("setting", "set_size5", self.setting.set_size[2][0])
            ini.write_int("setting", "set_size6", self.setting.set_size[2][1])
            ini.write_int("setting", "aspect_rate0", self.setting.aspect_rate[0])
            ini.write_int("setting", "aspect_rate1", self.setting.aspect_rate[1])
            ini.save()
        except Exception, s:
            self.messagebox("%s" % s)
        gtk.main_quit()

    def set_play_position(self, location):
        event = gst.event_new_seek(
                1.0,
                gst.FORMAT_TIME,
                gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
                gst.SEEK_TYPE_SET,
                location,
                gst.SEEK_TYPE_NONE,
                0)
        self.player.send_event(event)

    def clear(self):
        self.p_position = 0
        self.is_playing = False;
        if self.timer_id != -1:
            gobject.source_remove(self.timer_id)
            self.timer_id = -1
        if self.p_duration != gst.CLOCK_TIME_NONE:
            self.player.set_state(gst.STATE_READY)
            self.toolbox.seekadj.set_value(0)
            allsec = self.p_duration / 1000000000
            secv = "00:00/%02d:%02d" % ((allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)
            self.toolbox.change_img(1)
        self.video_window.set_flags(gtk.DOUBLE_BUFFERED)
        self.video_window.window.set_cursor(None)

    def change_video_size(self):
        val = self.settingwin.video_size
        # set size は全部処理、それ以外は初回は何もしない
        if val < 7:
            if self.player.get_property("uri") == None: # First start
                return
        self.w.set_focus(self.video_window)
        y = self.vsize.height
        asp = 1.00
        # Vertical reference
        if self.settingwin.aspect == 0:
            asp = self.vsize.width
        elif self.settingwin.aspect == 1:
            asp = self.vsize.height * 4.0 / 3
        elif self.settingwin.aspect == 2:
            asp = self.vsize.height * 16.0 / 9
        elif self.settingwin.aspect == 3:
            asp = self.vsize.height * self.setting.aspect_rate[0] / self.setting.aspect_rate[1]
        # aspect rate change
        self.aframe.set(xalign=0.5, yalign=0.5, ratio=float(asp)/float(y), obey_child=False)
        # window size @ return
        if val == 0:
            return
        x = int(asp)
        # 計算
        menu_v = self.menubar.allocation.height
        ctrl_v = self.toolbox.allocation.height
        stat_v = self.statusbar.allocation.height
        # 設定により振り分け
        if val == 2: # x2.0
            x *= 2
            y *= 2
        elif val == 3: # x3.0
            x *= 3
            y *= 3
        elif val == 4: # x0.5
            x /= 2
            y /= 2
        elif val == 5: # x1.5
            x = x * 3 / 2
            y = y * 3 / 2
        elif val == 6: # x2.5
            x = x * 5 / 2
            y = y * 5 / 2
        elif val == 7: # set1
            x = self.setting.set_size[0][0]
            y = self.setting.set_size[0][1]
        elif val == 8: # set2
            x = self.setting.set_size[1][0]
            y = self.setting.set_size[1][1]
        elif val == 9: # set3
            x = self.setting.set_size[2][0]
            y = self.setting.set_size[2][1]
        if x ==0 or y == 0:
            return
        # メニューやコントローラのサイズを加算
        # リストは表示されている場合のみ
        if self.listbox.get_property("visible"):
            list_h = self.listbox.get_property("width_request")
            x += list_h
        y += menu_v + ctrl_v + stat_v
        # 現在のサイズと違っていたらリサイズさせる、整数にしないと警告になる
        cx, cy = self.w.get_size()
        if cx != x or cy != y:
            self.w.resize(x, y)

    def hand_resize(self): #pass
        if self.settingwin.video_size != 0:
            self.uimanager.get_widget("/MenuBar/View/x00").set_active(True)

    def listup_change(self):
        uri = self.player.get_property("uri")
        if uri == None:
            return
        t = urllib.unquote(uri)
        p_path, self.p_filename = os.path.split(t)
        # パスが変わっていたら or リストにファイル名無しなら読み直し
        if self.p_path != p_path[7:] or self.listbox.is_text(self.p_filename) == False:
            self.p_path = p_path[7:]
            self.listbox.liststore.clear()
            files = numsort.delete_dotfile(os.listdir(self.p_path))
            numsort.sort_nicely(files)
            # リストに追加された数を数える
            self.rlist.clear()
            for f in files:
                # isfile
                name = os.path.join(self.p_path, f)
                if os.path.isfile(name):
                    # MIME
                    name_uri = urllib.quote(name)
                    mime = gnomevfs.get_mime_type(name_uri)
                    # MIME 登録されていなくても一致ファイルだけは追加
                    if mime in self.mime_list or f == self.p_filename:
                        it = self.listbox.liststore.append([f])
                        self.rlist.append(f)
                        # セレクタの変更
                        if f == self.p_filename:
                            self.listbox.select_text(self.p_filename)
        else:
            # 同じディレクトリにあるファイルをドロップ
            # なんて場合があるので一応変更しておく
            self.listbox.select_text(self.p_filename)
        self.w.set_title("%s (%d/%d) - Y901x" % (
                self.p_filename,
                self.listbox.get_select_num() + 1,
                len(self.listbox.liststore))
            )
        # random 順なリスト作成
        self.rlist.shuffle()
        self.listbox.selection_scroll()

    def rename(self):
        if self.p_filename == "":
            return
        label = gtk.Label(self.p_filename)
        entry = gtk.Entry()
        entry.set_text(self.p_filename)
        d = gtk.Dialog( "リネーム",
                        self.w,
                        gtk.DIALOG_MODAL,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                        gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
        try:
            d.vbox.pack_start(label, False)
            d.vbox.pack_start(entry, False)
            d.show_all()
            def dlg_ok(self):
                d.response(gtk.RESPONSE_ACCEPT)
            def on_focus_in(self, widget):
                try:
                    # calc the UNICODE
                    s = unicode(entry.get_text())
                    i = s.rindex(".")
                    entry.select_region(0, i)
                except ValueError:
                    # 何もしなければ全選択になる
                    pass
            def on_focus_out(self, widget):
                entry.select_region(0, 0)
            entry.connect("activate", dlg_ok)
            entry.connect("focus-in-event", on_focus_in)
            entry.connect("focus-out-event", on_focus_out)
            # 再生中ならポーズ、リピート中だと次に行ってしまう
            is_play = False
            if self.player.get_state(1)[1] == gst.STATE_PLAYING:
                is_play = True
                self.player.set_state(gst.STATE_PAUSED)
            # Until the Success or Cancel.
            while 1:
                if d.run() == gtk.RESPONSE_ACCEPT:
                    text = entry.get_text()
                    if text == self.p_filename:
                        self.messagebox("変更されていません")
                    else:
                        if  text in os.listdir(self.p_path):
                            self.messagebox("同一ファイル名が見つかりました")
                        else:
                            newname = os.path.join(self.p_path, text)
                            self.clear()
                            while gtk.events_pending():
                                gtk.main_iteration()
                            os.rename(os.path.join(self.p_path, self.p_filename), newname)
                            while gtk.events_pending():
                                gtk.main_iteration()
                            self.set_uri("file://" + newname)
                            return
                else:
                    break
        finally:
            d.destroy()
        if is_play:
            self.player.set_state(gst.STATE_PLAYING)

    def messagebox_trash(self, text):
        dlg = gtk.MessageDialog(self.w, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, 
                                gtk.BUTTONS_NONE, text)
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button("ゴミ箱へ移動する", gtk.RESPONSE_OK)
        dlg.set_default_response(gtk.RESPONSE_OK)
        dlg.set_title("確認 - Y901x")  
        r = dlg.run()  
        dlg.destroy()
        return r

    def delete(self):
        if self.p_filename == "":
            return
        # 再生中ならポーズ、リピート中だと次に行ってしまう
        is_play = False
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            is_play = True
            self.player.set_state(gst.STATE_PAUSED)
        r = self.messagebox_trash("再生中ファイルをゴミ箱に捨てます\nよろしいですか？")
        if r == gtk.RESPONSE_OK:
            fname = self.get_next_list()
            newname = os.path.join(self.p_path, fname)
            self.clear()
            while gtk.events_pending():
                gtk.main_iteration()
            # Trash
            obj = gio.File(os.path.join(self.p_path, self.p_filename))
            try:
                res = obj.trash()
            except Exception, s:
                self.messagebox("%s" % s)
                res = False
            if res:
                while gtk.events_pending():
                    gtk.main_iteration()
                self.p_path = ""
                if fname != "":
                    self.set_uri("file://" + newname)
                else:
                    self.p_filename = ""
                    self.listbox.liststore.clear()
                return
        if is_play:
            self.player.set_state(gst.STATE_PLAYING)

    def delete_next(self):
        if self.p_filename == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            it2 = model.iter_next(it)
            if it2:
                fname = model.get_value(it2, 0)
                delname = os.path.join(self.p_path, fname)
                obj = gio.File(os.path.join(delname))
                try:
                    res = obj.trash()
                    self.p_path = ""
                    self.listup_change()
                except Exception, s:
                    self.messagebox("%s" % s)

    def delete_prev(self):
        if self.p_filename == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            row = model.get_path(it)[0] - 1
            if row > -1:
                it2 = model.get_iter((row, ))
                fname = model.get_value(it2, 0)
                delname = os.path.join(self.p_path, fname)
                obj = gio.File(os.path.join(delname))
                try:
                    res = obj.trash()
                    self.p_path = ""
                    self.listup_change()
                except Exception, s:
                    self.messagebox("%s" % s)

    def get_next_list(self):
        s = ""
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return s
        # next
        it2 = model.iter_next(it)
        if it2 == None:
            # no next
            row = model.get_path(it)[0] - 1
            if row < 0:
                return s
            self.listbox.selection.select_path(row)
            it3 = self.listbox.selection.get_selected()[1]
            s = model.get_value(it3, 0)
        else:
            self.listbox.selection.select_iter(it2)
            s = model.get_value(it2, 0)
        return s

    def change_fullscreen(self):
        if self.p_filename == "":
            return
        if self.fullscreen:
            self.fullscreen = False
            self.w.window.unfullscreen()
            self.w.unfullscreen()
            self.fullobj.destroy()
            self.fullobj = None
            self.menubar.show()
            if self.full_list:
                self.listbox.show()
            self.w.resize(self.fullsize.width, self.fullsize.height)
        else:
            self.fullscreen = True
            cx, cy = self.w.get_size()
            self.fullsize.width = cx
            self.fullsize.height = cy
            self.full_list = self.listbox.get_property("visible")
            if self.full_list:
                self.listbox.hide()
            self.menubar.hide()
            self.fullobj = CFullCtrl(
                    self.hbox_ctrl,
                    self.toolbox,
                    self.vbox_main,
                    self.statusbar
                )
            self.w.fullscreen()
            self.w.window.fullscreen()


########################################################################
#
#                   Menu & Button Signal Handlers
#
########################################################################


    def on_open(self, widget, event=None):
        dlg = gtk.FileChooserDialog("ファイルを開く", self.w,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        # Set MIME Types Filter
        filt = gtk.FileFilter()
        filt.set_name("Support file")
        for mimestring in self.mime_list:
            filt.add_mime_type(mimestring)
        dlg.add_filter(filt)
        filt = gtk.FileFilter()
        filt.set_name("All files")
        filt.add_pattern("*")
        dlg.add_filter(filt)
        # Set Open Directry
        if self.p_path == "":
            # search VIDEOS Directry
            sd = glib.get_user_special_dir(glib.USER_DIRECTORY_VIDEOS)
            if sd:
                dlg.set_current_folder(sd)
            else:
                dlg.set_current_folder(os.environ["HOME"])
        else:
            dlg.set_current_folder(self.p_path)
        r = dlg.run()
        if r == gtk.RESPONSE_OK:
            self.set_uri("file://" + dlg.get_filename())
        dlg.destroy()

    def on_play(self, widget, event=None):
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            self.player.set_state(gst.STATE_PAUSED)
        else:
            if self.player.set_state(gst.STATE_PLAYING) == gst.STATE_CHANGE_FAILURE:
                self.messaagebox("再生できませんでした")

    def on_stop(self, widget, event=None):
        self.clear()

    def on_prev(self, widget, event=None):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(gst.STATE_PLAYING)
            else:
                self.set_uri("file://%s/%s" % (self.p_path, self.rlist.get()))
        else:
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            row = model.get_path(it)[0] - 1
            if row < 0:
                return
            # Change Listbox Select
            self.listbox.selection.select_path(row)
            it = self.listbox.selection.get_selected()[1]
            s = model.get_value(it, 0)
            self.set_uri("file://%s/%s" % (self.p_path, s))

    def on_next(self, widget, event=None):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(gst.STATE_PLAYING)
            else:
                self.set_uri("file://%s/%s" % (self.p_path, self.rlist.get()))
        else:
            # is endressloop ?
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            it = model.iter_next(it)
            if it == None:
                if self.setting.endressloop:
                    it = model.get_iter_first()
                else:
                    return
            # Change Listbox Select
            self.listbox.selection.select_iter(it)
            s = model.get_value(it, 0)
            self.set_uri("file://%s/%s" % (self.p_path, s))

    def on_rep_change(self, action, current):
        num = action.get_current_value()
        self.settingwin.repeat = num
        self.on_rep_btn(None, num)

    def on_rep_btn(self, widget, data):
        # widget==None is Escape
        if widget == None:
            if data == 0:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(False)
            elif data == 3:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(True)
            elif data == 2:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(True)
                self.toolbox.rdm.set_active(False)
            elif data == 1:
                self.toolbox.one.set_active(True)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(False)
        else:
            if widget.get_active():
                if self.settingwin.repeat != data:
                    action = self.actiongroup.get_action(ac2_str[data])
                    action.set_current_value(data)
                    if data != 3:
                        self.toolbox.rdm.set_active(False)
                    if data != 2:
                        self.toolbox.all.set_active(False)
                    if data != 1:
                        self.toolbox.one.set_active(False)
            else:
                if self.settingwin.repeat == data:
                    action = self.actiongroup.get_action(ac2_str[0])
                    action.set_current_value(0)
                    self.toolbox.one.set_active(False)
                    self.toolbox.all.set_active(False)
                    self.toolbox.rdm.set_active(False)

    def on_asp_change(self, action, current):
        num = action.get_current_value()
        self.settingwin.aspect = num
        if num == 3:
            s = "%d:%d" % (self.setting.aspect_rate[0], self.setting.aspect_rate[1])
            self.statusbar.label[2].set_text(s)
        else:
            self.statusbar.label[2].set_text(asp_str[num])
        self.change_video_size()

    def on_quit(self, widget, event=None):
        self.close()
        return True

    def on_full(self, widget, event=None):
        self.change_fullscreen()

    def on_size_change(self, action, current):
        # Release FullScreen
        if self.fullscreen:
            self.change_fullscreen()
        # Draw StatusBar Text
        num = action.get_current_value()
        self.settingwin.video_size =  num
        self.statusbar.label[1].set_text(size_str[num])
        self.change_video_size()

    def on_setting(self, widget, event=None):
        dlg = y901dialog.SettingDialog(self.w, self.setting)
        try:
            if dlg.run() == gtk.RESPONSE_ACCEPT:
                self.setting.position = dlg.cbox_pos.get_active()
                self.setting.init_size_on = dlg.cbox_size.get_active()
                self.setting.init_size_val = dlg.combo_size.get_active()
                self.setting.esc_exit = dlg.cbox_esc.get_active()
                self.setting.mem_rep = dlg.cbox_memrep.get_active()
                self.setting.mem_aspect = dlg.cbox_memaspect.get_active()
                self.setting.endressloop = dlg.cbox_loop.get_active()
                self.setting.severity = int(dlg.sb_mouse.get_value())
                startiter = dlg.mimebuf.get_start_iter()
                enditer   = dlg.mimebuf.get_end_iter()
                mimes = dlg.mimebuf.get_text(startiter, enditer).replace("\n", "|")
                if mimes != self.setting.mimes:
                    self.setting.mimes = mimes
                    self.mime_list = mimes.split("|")
                    self.p_path = ""
                    self.listup_change()
                self.setting.set_size[0][0] = int(dlg.sb[0].get_value())
                self.setting.set_size[0][1] = int(dlg.sb[1].get_value())
                self.setting.set_size[1][0] = int(dlg.sb[2].get_value())
                self.setting.set_size[1][1] = int(dlg.sb[3].get_value())
                self.setting.set_size[2][0] = int(dlg.sb[4].get_value())
                self.setting.set_size[2][1] = int(dlg.sb[5].get_value())
                aspect = dlg.entry_aspect.get_text().split(":")
                self.setting.aspect_rate[0] = int(aspect[0])
                self.setting.aspect_rate[1] = int(aspect[1])
        except Exception, s:
            self.messagebox("%s" % s)
        finally:
            dlg.destroy()
        self.w.set_focus(self.video_window)

    def on_about(self, widget, event=None):
        url = "http://palepoli.skr.jp/"
        w = gtk.AboutDialog()
        w.set_transient_for(self.w)
        w.set_program_name("Y901x")
        w.set_version(version)
        w.set_copyright("Copyright(C) 2009-2010 sasakima-nao")
        w.set_license("GPL")
        w.set_logo(self.icon)
        # comments
        v = "GTK+ Version %d.%d.%d\n" % gtk.ver
        v += "PyGtk Version %d.%d.%d" % gtk.pygtk_version
        w.set_comments(v)
        # Order set_url_fook set_website
        def about_url(about, link, data): os.system("xdg-open %s" % link)
        gtk.about_dialog_set_url_hook(about_url, url)
        w.set_website(url)
        w.set_website_label(url)
        w.run()
        w.destroy()

    def on_list_button_clicked(self, widget, event=None):
        if self.fullscreen:
            return
        width = self.listbox.get_property("width_request")
        cx, cy = self.w.get_size()
        if self.listbox.get_property("visible"):
            self.listbox.hide()
            self.w.resize(cx - width, cy)
        else:
            self.listbox.show()
            self.w.resize(cx + width, cy)


########################################################################
#
#                   MainWindow Signal Handlers
#
########################################################################

    
    def on_button_press(self, widget, event):
        # Start Gesture
        if event.button == 3:
            self.gesture.start = True
            self.gesture.x = event.x_root
            self.gesture.y = event.y_root
            self.gesture.result = ""
            self.gesture.prevchar = ""
            self.gesture.title = ""
        elif event.type == gtk.gdk._2BUTTON_PRESS:
            self.change_fullscreen()
    
    def on_motion_notify(self, widget, event):
        # 
        if self.gesture.start:
            x = abs(event.x_root - self.gesture.x)
            y = abs(event.y_root - self.gesture.y)
            s = ""
            if (x > self.setting.severity) or (y > self.setting.severity):
                if x > y:
                    if (self.gesture.x - event.x_root) > 0:
                        s = "←"
                    else:
                        s = "→"
                else:
                    if (self.gesture.y - event.y_root) > 0:
                        s = "↑"
                    else:
                        s = "↓"
                if self.gesture.prevchar != s:
                    self.gesture.prevchar = s
                    self.gesture.result += s
                self.gesture.x = event.x_root
                self.gesture.y = event.y_root
                if self.gesture.title == "":
                    self.gesture.title = self.w.get_title()
                self.w.set_title(self.gesture.result)
    
    def on_button_release(self, widget, event):
        # gesture check
        # if event.button == 3: is no
        if self.gesture.start:
            self.gesture.start = False
            # Separation (Prevent PopupMenu)
            if self.gesture.result == "" and self.w.get_title() != "":
                self.popup_menu.popup(None, None, None, 0, event.time)
            else:
                if self.gesture.title != "":
                    self.w.set_title(self.gesture.title)
                if self.gesture.result == "→":
                    self.on_next(widget,0)
                elif self.gesture.result == "←":
                    self.on_prev(widget,0)
                elif self.gesture.result == "↓→":
                    self.close()
        return True

    def on_drag_data_received(self, widget, context, x, y, selection_data, info, time):
        # D&D FileNames split CRLF or LF
        if "\r\n" in selection_data.data:
            drop = selection_data.data.split("\r\n")[0]
        else:
            drop = selection_data.data.split("\n")[0]
        if "//" in drop:
            self.set_uri(drop)
        else:
            self.set_uri("file://" + drop)

    def on_key_press(self, widget, event):
        # Mix gtk.gdk.MOD2_MASK
        if event.state == (gtk.gdk.CONTROL_MASK | gtk.gdk.MOD2_MASK):
            val = self.toolbox.volumebar.get_value()
            if event.keyval == gtk.keysyms.Up:
                if val < 100:
                    self.toolbox.volumebar.set_value(val + 1)
                return True
            elif event.keyval == gtk.keysyms.Down:
                if val > 0:
                    self.toolbox.volumebar.set_value(val - 1)
                return True
        else:
            if event.keyval == gtk.keysyms.Up:
                self.on_prev(widget)
                return True
            elif event.keyval == gtk.keysyms.Down:
                self.on_next(widget)
                return True

    def on_key_release(self, widget, event):
        self.w.set_focus(self.video_window)


########################################################################
#
#                       About Signal Handlers
#
########################################################################


    def on_timer(self):
        try:
            self.p_position = self.player.query_position(gst.FORMAT_TIME)[0]
        except:
            self.p_position = gst.CLOCK_TIME_NONE
        if self.p_position != gst.CLOCK_TIME_NONE:
            self.toolbox.seekadj.set_value(self.p_position)
            sec = self.p_position / 1000000000
            allsec = self.p_duration / 1000000000
            secv = "%02d:%02d/%02d:%02d" % ((sec / 60), (sec % 60), (allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)
        # GetCursolPos
        a, x, y, b = gtk.gdk.display_get_default().get_pointer()
        # FullScreen at Controler
        if self.fullobj:
            if self.fullobj.y_pos < y:
                self.fullobj.show_all()
            else:
                self.fullobj.hide()
        # Cursor Auto Invisible
        if self.mousepoint.x == x and self.mousepoint.y == y:
            if self.invisiblecount < 5:
                self.invisiblecount += 1
            else:
                self.video_window.window.set_cursor(self.noncursor)
        else:
            self.invisiblecount = 0;
            self.mousepoint.x = x
            self.mousepoint.y = y
            self.video_window.window.set_cursor(None)
        # return True is Repeat
        return self.is_playing

    def on_seek_value_changed(self, widget, event=None):
        if self.timer_id == -1:
            return
        if self.is_seek:
            location = widget.get_value()
            self.set_play_position(location)
            sec = location / 1000000000
            allsec = self.p_duration / 1000000000
            secv = "%02d:%02d/%02d:%02d" % ((sec / 60), (sec % 60), (allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)

    def on_seek_button_press_event(self, widget, event):
        if self.timer_id == -1:
            return
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            self.player.set_state(gst.STATE_PAUSED)
            self.seek_at_play = True
        gobject.source_remove(self.timer_id)
        self.is_seek = True
        event.button = 2

    def on_seek_button_release_event(self, widget, event):
        self.w.set_focus(self.video_window)
        if self.timer_id == -1:
            return
        self.is_seek = False
        if self.seek_at_play:
            self.seek_at_play = False
            self.player.set_state(gst.STATE_PLAYING)
        self.timer_id = gobject.timeout_add(200, self.on_timer)
        event.button = 2

    def set_volume(self, value):
        v = gst.interfaces.stream_volume_convert_volume(
                gst.interfaces.STREAM_VOLUME_FORMAT_CUBIC,
                gst.interfaces.STREAM_VOLUME_FORMAT_LINEAR,
                value)
        v = clamp(v, 0.0, 1.0)
        if self.player.get_property("volume") != v:
            self.player.set_property("volume", v)

    def on_volume_value_changed(self, widget, event=None):
        self.set_volume(widget.get_value() / 100.0)

    def on_volume_button_release_event(self, widget, event):
        self.w.set_focus(self.video_window)

    def on_listbox_button_press(self, widget, event=None):
        if event.button == 3:
            self.listpopup.popup(None, None, None, 0, event.time)
            return True

    def on_listbox_button_release(self, widget, event=None):
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return
        s = model.get_value(it, 0)
        if s != self.p_filename:
            new_uri = "file://" + self.p_path + "/" + s
            self.set_uri(new_uri)
        self.w.set_focus(self.video_window)

    def on_aframe_expose(self, widget, event):
        gc = widget.style.black_gc
        widget.window.draw_rectangle(gc, True, event.area.x, event.area.y,
                                        event.area.width, event.area.height)

    def on_keypad(self, accelGroup, window, keyval, modifier):
        # テンキー処理
        g = None
        if keyval == 65456:
            g = self.uimanager.get_widget("/MenuBar/View/x00")
        elif keyval == 65457:
            g = self.uimanager.get_widget("/MenuBar/View/x10")
        elif keyval == 65458:
            g = self.uimanager.get_widget("/MenuBar/View/x20")
        elif keyval == 65459:
            g = self.uimanager.get_widget("/MenuBar/View/x30")
        elif keyval == 65460:
            g = self.uimanager.get_widget("/MenuBar/View/x05")
        elif keyval == 65461:
            g = self.uimanager.get_widget("/MenuBar/View/x15")
        elif keyval == 65462:
            g = self.uimanager.get_widget("/MenuBar/View/x25")
        elif keyval == 65463:
            g = self.uimanager.get_widget("/MenuBar/View/xs1")
        elif keyval == 65464:
            g = self.uimanager.get_widget("/MenuBar/View/xs2")
        elif keyval == 65465:
            g = self.uimanager.get_widget("/MenuBar/View/xs3")
        # exec on_size_change signal
        g.set_active(True)

    def on_keydown(self, accelGroup, window, keyval, modifier):
        # Non Menu Keybord
        if keyval == gtk.keysyms.F12:
            self.on_list_button_clicked(window)
        elif keyval == gtk.keysyms.F5:
            self.p_path = ""
            self.listup_change()
        elif keyval == gtk.keysyms.F2:
            self.rename()
        elif keyval == gtk.keysyms.Escape:
            if self.fullscreen:
                self.change_fullscreen()
            elif self.setting.esc_exit:
                self.close()
        elif keyval == gtk.keysyms.Delete:
            if modifier == gtk.gdk.SHIFT_MASK:
                self.delete_prev()
            elif modifier == gtk.gdk.CONTROL_MASK:
                self.delete_next()
            else:
                self.delete()


########################################################################
#
#                       GStreamer Event Handlers
#
########################################################################


    def gst_on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            self.clear()
            # Repeat
            if self.settingwin.repeat > 1:
                self.on_next(self)
            elif self.settingwin.repeat == 1:
                self.set_play_position(0)
                self.player.set_state(gst.STATE_PLAYING)
        elif t == gst.MESSAGE_ERROR:
            self.clear()
            err, debug = message.parse_error()
            self.messagebox(str(err) + "\n" + str(debug))  
            self.toolbox.change_img(0)
        elif t == gst.MESSAGE_STATE_CHANGED:
            if gst.STATE_PLAYING == message.parse_state_changed()[1]:
                self.toolbox.change_img(0)
                # This is First Play
                if not self.is_playing:
                    try:
                        self.p_duration = self.player.query_duration(gst.FORMAT_TIME)[0]
                    except:
                        self.p_duration = gst.CLOCK_TIME_NONE
                    if self.p_duration != gst.CLOCK_TIME_NONE:
                        self.toolbox.seekadj.upper = self.p_duration
                        self.is_playing = True
                        # toolbox Buttons Enabled
                        self.toolbox.set_enable_ctrl(True)
                        # Timer on
                        self.timer_id = gobject.timeout_add(200, self.on_timer)
                        # File Names Listup
                        self.listup_change()
                        #
                        # playbin2
                        # Audio
                        asink = self.player.get_property("audio-sink")
                        if asink:
                            self.toolbox.volumebar.set_property("sensitive", True)
                            v = self.toolbox.volumeadj.get_value() / 100.0
                            self.set_volume(v)
                        else:
                            self.toolbox.volumebar.set_property("sensitive", False)
                        # Video vsink == None @ Music File
                        vsink = self.player.get_property("video-sink")
                        if vsink:
                            for pad in vsink.pads():
                                # W Buffer Off
                                self.video_window.unset_flags(gtk.DOUBLE_BUFFERED)
                                # caps[0] @ GstStructure
                                caps = pad.get_negotiated_caps()
                                self.vsize.width = caps[0]["width"]
                                self.vsize.height = caps[0]["height"]
                                #
                                flt = float(self.vsize.width) / float(self.vsize.height)
                                self.aframe.set(xalign=0.5, yalign=0.5, ratio=flt, obey_child=False)
                                self.change_video_size()
            else:
                self.toolbox.change_img(1)

    def gst_on_sync_message(self, bus, message):
        if message.structure is None:
            return
        message_name = message.structure.get_name()
        if message_name == "prepare-xwindow-id":
            imagesink = message.src
            imagesink.set_property("force-aspect-ratio", False)
            imagesink.set_xwindow_id(self.video_window.window.xid)



########################################################################
#
#                               main
#
########################################################################


def init_window(bus_name):
    if "-V" in sys.argv:
        print(version)
        return 0
    w = Y901window(bus_name)
    #gtk.gdk.threads_init()
    gtk.main()

def main():
    try:
        # Open the SessionBus
        session_bus = dbus.SessionBus()
        # Check init Y901x. Not Found @ exception
        remote_object = session_bus.get_object("org.y901x.Service1", "/org/y901x/Object1")
        # Prepared notify_startup_complete
        while gtk.events_pending():
            gtk.main_iteration()
        # Send URI
        if len(sys.argv) > 1:
            if "//" in sys.argv[1]:
                uri = sys.argv[1]
            else:
                uri = "file://" + sys.argv[1]
            try:
                remote_object.set_uri(uri, dbus_interface = "org.y901x.Interface1")
            except:
                print("Y901x: Error remote_object.set_uri")
        # Done
        try:
            gtk.gdk.notify_startup_complete()
        except:
            print("Y901x: Error notify_startup_complete")
        return
    except:
        bus_name = dbus.service.BusName("org.y901x.Service1", bus=session_bus)
        init_window(bus_name)

if __name__ == "__main__":
    main()
